'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

exports.createCanvas = createCanvas;
exports.createCanvasFromImageBuffer = createCanvasFromImageBuffer;
exports.cloneCanvas = cloneCanvas;
exports.drawPixel = drawPixel;
exports.drawRect = drawRect;
exports.drawCanvas = drawCanvas;
exports.replaceColor = replaceColor;
exports.getColor = getColor;

var _utils = require('./utils.js');

var _constants = require('./constants.js');

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

/**
 * Creates a new empty canvas.
 */
function createCanvas(width, height) {
	var hasAlphaChannel = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : true;

	var channels = hasAlphaChannel ? _constants.RGBA : _constants.RGB;

	return {
		width: width,
		height: height,
		hasAlphaChannel: hasAlphaChannel,
		data: new Uint8Array(width * height * channels).fill(0x00)
	};
}

/**
 * Creates a new canvas object from an image buffer.
 */
function createCanvasFromImageBuffer(imageBuffer, width, height) {
	var hasAlphaChannel = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : true;

	var canvas = createCanvas(width, height, hasAlphaChannel);
	canvas.data = [].concat(_toConsumableArray(imageBuffer));
	return canvas;
}

/**
 * Clones the canvas object.
 */
function cloneCanvas(canvas) {
	return createCanvasFromImageBuffer(canvas.data, canvas.width, canvas.height, canvas.hasAlphaChannel);
}

/**
 * Sets one pixels color on the canvas.
 */
function drawPixel(canvas, x, y, color) {
	var workingCanvas = cloneCanvas(canvas),
	    bytePos = (0, _utils.coordinates2bytePosition)(canvas, x, y);

	var _color = _slicedToArray(color, 3);

	workingCanvas.data[bytePos + _constants.CHANNEL_RED] = _color[0];
	workingCanvas.data[bytePos + _constants.CHANNEL_GREEN] = _color[1];
	workingCanvas.data[bytePos + _constants.CHANNEL_BLUE] = _color[2];


	if (color[_constants.CHANNEL_ALPHA] && canvas.hasAlphaChannel) {
		workingCanvas.data[bytePos + _constants.CHANNEL_ALPHA] = color[_constants.CHANNEL_ALPHA];
	}

	return workingCanvas;
}

/**
 * Draws a rectangle.
 */
function drawRect(canvas, x, y, width, height, color) {
	var workingCanvas = cloneCanvas(canvas);

	for (var i = x; i < x + width; i++) {
		for (var j = y; j < y + height; j++) {
			var bytePos = (0, _utils.coordinates2bytePosition)(canvas, i, j);

			var _color2 = _slicedToArray(color, 3);

			workingCanvas.data[bytePos + _constants.CHANNEL_RED] = _color2[0];
			workingCanvas.data[bytePos + _constants.CHANNEL_GREEN] = _color2[1];
			workingCanvas.data[bytePos + _constants.CHANNEL_BLUE] = _color2[2];


			if (canvas.hasAlphaChannel) {
				workingCanvas.data[bytePos + _constants.CHANNEL_ALPHA] = (0, _utils.isRGBA)(color) ? color[_constants.CHANNEL_ALPHA] : 0xff;
			}
		}
	}

	return workingCanvas;
}

/**
 * Draws a canvas on another canvas.
 */
function drawCanvas(destination, source, offsetX, offsetY) {
	var workingCanvas = cloneCanvas(destination);

	(0, _utils.forEachPixel)(source, function (x, y, bytePos) {
		if (!(0, _utils.hasCoordinates)(destination, x + offsetX, y + offsetY) || (0, _utils.getAlpha)(getColor(source, x, y)) === 0x00) {
			return;
		}

		var destBytePos = (0, _utils.coordinates2bytePosition)(destination, x + offsetX, y + offsetY),
		    alpha = 0xff;

		var _blendColor = (0, _utils.blendColor)(getColor(destination, x, y), getColor(source, x, y));

		var _blendColor2 = _slicedToArray(_blendColor, 4);

		workingCanvas.data[destBytePos + _constants.CHANNEL_RED] = _blendColor2[0];
		workingCanvas.data[destBytePos + _constants.CHANNEL_GREEN] = _blendColor2[1];
		workingCanvas.data[destBytePos + _constants.CHANNEL_BLUE] = _blendColor2[2];
		alpha = _blendColor2[3];


		if (destination.hasAlphaChannel && source.hasAlphaChannel) {
			workingCanvas.data[destBytePos + _constants.CHANNEL_ALPHA] = alpha;
		}
	});

	return workingCanvas;
}

/**
 * Replaces the specified color on the canvas.
 */
function replaceColor(canvas, replacee, replacer) {
	var workingCanvas = cloneCanvas(canvas);

	(0, _utils.forEachPixel)(canvas, function (x, y, bytePos) {
		if ((0, _utils.isEqualColor)(getColor(canvas, x, y), replacee)) {
			var _replacer = _slicedToArray(replacer, 3);

			workingCanvas.data[bytePos + _constants.CHANNEL_RED] = _replacer[0];
			workingCanvas.data[bytePos + _constants.CHANNEL_GREEN] = _replacer[1];
			workingCanvas.data[bytePos + _constants.CHANNEL_BLUE] = _replacer[2];
		}
	});

	return workingCanvas;
}

/**
 * Returns with the color of the specified coordinates.
 */
function getColor(canvas, x, y) {
	var bytePos = (0, _utils.coordinates2bytePosition)(canvas, x, y),
	    color = [canvas.data[bytePos + _constants.CHANNEL_RED], canvas.data[bytePos + _constants.CHANNEL_GREEN], canvas.data[bytePos + _constants.CHANNEL_BLUE]];

	if (canvas.hasAlphaChannel) {
		color[_constants.CHANNEL_ALPHA] = canvas.data[bytePos + _constants.CHANNEL_ALPHA];
	}

	return color;
}